@@ -6,7 +6,7 @@ using Base.BinaryPlatforms: HostPlatform, Platform, platforms_match
 using Test
 using TOML
 
-ENV["HISTORICAL_STDLIB_VERSIONS_AUTO_REGISTER"]="false"
+ENV["HISTORICAL_STDLIB_VERSIONS_AUTO_REGISTER"] = "false"
 using HistoricalStdlibVersions
 
 include("utils.jl")
@@ -70,9 +70,9 @@ end
         return only(deps[name])
     end
 
-    isolate(loaded_depot=true) do
+    isolate(loaded_depot = true) do
         # Next, test that if we ask for `v1.5` it DOES have a version, and that GMP_jll installs v6.1.X
-        Pkg.add(["NetworkOptions", "GMP_jll"]; julia_version=v"1.5")
+        Pkg.add(["NetworkOptions", "GMP_jll"]; julia_version = v"1.5")
         no_block = get_manifest_block("NetworkOptions")
         @test haskey(no_block, "uuid")
         @test no_block["uuid"] == networkoptions_uuid
@@ -118,7 +118,7 @@ end
 
     # Next, test that if we ask for `v1.6`, GMP_jll gets `v6.2.0`, and for `v1.7`, it gets `v6.2.1`
     function do_gmp_test(julia_version, gmp_version)
-        isolate(loaded_depot=true) do
+        isolate(loaded_depot = true) do
             Pkg.add("GMP_jll"; julia_version)
             gmp_block = get_manifest_block("GMP_jll")
             @test haskey(gmp_block, "uuid")
@@ -130,9 +130,9 @@ end
     do_gmp_test(v"1.6", v"6.2.0")
     do_gmp_test(v"1.7", v"6.2.1")
 
-    isolate(loaded_depot=true) do
+    isolate(loaded_depot = true) do
         # Next, test that if we ask for `nothing`, NetworkOptions has a `version` but `LinearAlgebra` does not.
-        Pkg.add(["LinearAlgebra", "NetworkOptions"]; julia_version=nothing)
+        Pkg.add(["LinearAlgebra", "NetworkOptions"]; julia_version = nothing)
         no_block = get_manifest_block("NetworkOptions")
         @test haskey(no_block, "uuid")
         @test no_block["uuid"] == networkoptions_uuid
@@ -143,7 +143,7 @@ end
         @test !haskey(linalg_block, "version")
     end
 
-    isolate(loaded_depot=true) do
+    isolate(loaded_depot = true) do
         # Next, test that stdlibs do not get dependencies from the registry
         # NOTE: this test depends on the fact that in Julia v1.6+ we added
         # "fake" JLLs that do not depend on Pkg while the "normal" p7zip_jll does.
@@ -170,30 +170,36 @@ end
 
         # First, we're going to resolve for specific versions of Julia, ensuring we get the right dep versions:
         Pkg.Registry.download_default_registries(Pkg.stdout_f())
-        ctx = Pkg.Types.Context(;julia_version=v"1.5")
-        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
-            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
-        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
+        ctx = Pkg.Types.Context(; julia_version = v"1.5")
+        versions, deps = Pkg.Operations._resolve(
+            ctx.io, ctx.env, ctx.registries, [
+                Pkg.Types.PackageSpec(name = "MPFR_jll", uuid = Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
+            ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version
+        )
         gmp = find_by_name(versions, "GMP_jll")
         @test gmp !== nothing
         @test gmp.version.major == 6 && gmp.version.minor == 1
-        ctx = Pkg.Types.Context(;julia_version=v"1.6")
-        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
-            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
-        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
+        ctx = Pkg.Types.Context(; julia_version = v"1.6")
+        versions, deps = Pkg.Operations._resolve(
+            ctx.io, ctx.env, ctx.registries, [
+                Pkg.Types.PackageSpec(name = "MPFR_jll", uuid = Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
+            ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version
+        )
         gmp = find_by_name(versions, "GMP_jll")
         @test gmp !== nothing
         @test gmp.version.major == 6 && gmp.version.minor == 2
 
         # We'll also test resolving an "impossible" manifest; one that requires two package versions that
         # are not both loadable by the same Julia:
-        ctx = Pkg.Types.Context(;julia_version=nothing)
-        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
-            # This version of GMP only works on Julia v1.6
-            Pkg.Types.PackageSpec(name="GMP_jll", uuid=Base.UUID("781609d7-10c4-51f6-84f2-b8444358ff6d"), version=v"6.2.0"),
-            # This version of MPFR only works on Julia v1.5
-            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3"), version=v"4.0.2"),
-        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
+        ctx = Pkg.Types.Context(; julia_version = nothing)
+        versions, deps = Pkg.Operations._resolve(
+            ctx.io, ctx.env, ctx.registries, [
+                # This version of GMP only works on Julia v1.6
+                Pkg.Types.PackageSpec(name = "GMP_jll", uuid = Base.UUID("781609d7-10c4-51f6-84f2-b8444358ff6d"), version = v"6.2.0+0"),
+                # This version of MPFR only works on Julia v1.5
+                Pkg.Types.PackageSpec(name = "MPFR_jll", uuid = Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3"), version = v"4.0.2+0"),
+            ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version
+        )
         gmp = find_by_name(versions, "GMP_jll")
         @test gmp !== nothing
         @test gmp.version.major == 6 && gmp.version.minor == 2
@@ -210,44 +216,44 @@ OpenBLAS_jll_UUID = Base.UUID("4536629a-c528-5b80-bd46-f80d51c5b363")
 libcxxwrap_julia_jll_UUID = Base.UUID("3eaa8342-bff7-56a5-9981-c04077f7cee7")
 libblastrampoline_jll_UUID = Base.UUID("8e850b90-86db-534c-a0d3-1478176c7d93")
 
-isolate(loaded_depot=true) do
+isolate(loaded_depot = true) do
     @testset "Elliot and Mosè's mini Pkg test suite" begin # https://github.com/JuliaPackaging/JLLPrefixes.jl/issues/6
         HistoricalStdlibVersions.register!()
         @testset "Standard add" begin
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Standard add (non-stdlib, flexible version)
-            Pkg.add(; name="HelloWorldC_jll")
+            Pkg.add(; name = "HelloWorldC_jll")
             @test haskey(Pkg.dependencies(), HelloWorldC_jll_UUID)
 
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Standard add (non-stdlib, url and rev)
-            Pkg.add(; name="HelloWorldC_jll", url="https://github.com/JuliaBinaryWrappers/HelloWorldC_jll.jl", rev="0b4959a49385d4bb00efd281447dc19348ebac08")
+            Pkg.add(; name = "HelloWorldC_jll", url = "https://github.com/JuliaBinaryWrappers/HelloWorldC_jll.jl", rev = "0b4959a49385d4bb00efd281447dc19348ebac08")
             @test Pkg.dependencies()[Base.UUID("dca1746e-5efc-54fc-8249-22745bc95a49")].git_revision === "0b4959a49385d4bb00efd281447dc19348ebac08"
 
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Standard add (non-stdlib, specified version)
-            Pkg.add(; name="HelloWorldC_jll", version=v"1.0.10+1")
+            Pkg.add(; name = "HelloWorldC_jll", version = v"1.0.10+1")
             @test Pkg.dependencies()[Base.UUID("dca1746e-5efc-54fc-8249-22745bc95a49")].version === v"1.0.10+1"
 
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Standard add (non-stdlib, versionspec)
-            Pkg.add(; name="HelloWorldC_jll", version=Pkg.Types.VersionSpec("1.0.10"))
+            Pkg.add(; name = "HelloWorldC_jll", version = Pkg.Types.VersionSpec("1.0.10"))
             @test Pkg.dependencies()[Base.UUID("dca1746e-5efc-54fc-8249-22745bc95a49")].version === v"1.0.10+1"
         end
 
         @testset "Julia-version-dependent add" begin
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Julia-version-dependent add (non-stdlib, flexible version)
-            Pkg.add(; name="libcxxwrap_julia_jll", julia_version=v"1.7")
+            Pkg.add(; name = "libcxxwrap_julia_jll", julia_version = v"1.7")
             @test Pkg.dependencies()[libcxxwrap_julia_jll_UUID].version >= v"0.14.0+0"
 
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Julia-version-dependent add (non-stdlib, specified version)
-            Pkg.add(; name="libcxxwrap_julia_jll", version=v"0.9.4+0", julia_version=v"1.7")
+            Pkg.add(; name = "libcxxwrap_julia_jll", version = v"0.9.4+0", julia_version = v"1.7")
             @test Pkg.dependencies()[libcxxwrap_julia_jll_UUID].version === v"0.9.4+0"
 
-            Pkg.activate(temp=true)
-            Pkg.add(; name="libcxxwrap_julia_jll", version=v"0.8.8+1", julia_version=v"1.9")
+            Pkg.activate(temp = true)
+            Pkg.add(; name = "libcxxwrap_julia_jll", version = v"0.8.8+1", julia_version = v"1.9")
             # FIXME? Pkg.dependencies() complains here that mbedtls_jll isn't installed so can't be used here.
             # Perhaps Pkg.dependencies() should just return state and not error if source isn't installed?
             @test_skip Pkg.dependencies()[libcxxwrap_julia_jll_UUID].version === v"0.9.4+0"
@@ -258,15 +264,20 @@ isolate(loaded_depot=true) do
             end
         end
 
+        @testset "Old Pkg add regression" begin
+            Pkg.activate(temp = true)
+            Pkg.add(; name = "Pkg", julia_version = v"1.11")
+        end
+
         @testset "Stdlib add" begin
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Stdlib add (current julia version)
-            Pkg.add(; name="GMP_jll")
+            Pkg.add(; name = "GMP_jll")
             @test Pkg.dependencies()[GMP_jll_UUID].version >= v"6.3.0+2" # v1.13.0-DEV
 
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Make sure the source of GMP_jll is installed
-            Pkg.add([PackageSpec("GMP_jll")]; julia_version=v"1.6")
+            Pkg.add([PackageSpec("GMP_jll")]; julia_version = v"1.6")
             src = Pkg.Operations.find_installed(
                 "GMP_jll",
                 Base.UUID("781609d7-10c4-51f6-84f2-b8444358ff6d"),
@@ -277,9 +288,9 @@ isolate(loaded_depot=true) do
             @test_broken isdir(src)
             @test_broken isfile(joinpath(src, "Artifacts.toml"))
 
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Stdlib add (other julia version)
-            Pkg.add(; name="GMP_jll", julia_version=v"1.7")
+            Pkg.add(; name = "GMP_jll", julia_version = v"1.7")
             @test Pkg.dependencies()[GMP_jll_UUID].version === v"6.2.1+1"
 
             # Stdlib add (other julia version, with specific version bound)
@@ -291,31 +302,72 @@ isolate(loaded_depot=true) do
             # then install that.  If we have to manually work around that and look up what
             # GMP_jll for Julia v1.7 is, then ask for that version explicitly, that's ok.
 
-            Pkg.activate(temp=true)
-            Pkg.add(; name="GMP_jll", julia_version=v"1.7")
+            Pkg.activate(temp = true)
+            Pkg.add(; name = "GMP_jll", julia_version = v"1.7")
 
             # This is expected to fail, that version can't live with `julia_version = v"1.7"`
-            @test_throws Pkg.Resolve.ResolverError Pkg.add(; name="GMP_jll", version=v"6.2.0+5", julia_version=v"1.7")
+            @test_throws Pkg.Resolve.ResolverError Pkg.add(; name = "GMP_jll", version = v"6.2.0+5", julia_version = v"1.7")
 
-            Pkg.activate(temp=true)
+            Pkg.activate(temp = true)
             # Stdlib add (julia_version == nothing)
-            # Note: this is currently known to be broken, we get the wrong GMP_jll!
-            Pkg.add(; name="GMP_jll", version=v"6.2.1+1", julia_version=nothing)
-            @test_broken Pkg.dependencies()[GMP_jll_UUID].version === v"6.2.1+1"
+            Pkg.add(; name = "GMP_jll", version = v"6.2.1+1", julia_version = nothing)
+            @test Pkg.dependencies()[GMP_jll_UUID].version === v"6.2.1+1"
         end
 
         @testset "julia_version = nothing" begin
-            Pkg.activate(temp=true)
-            # Stdlib add (impossible constraints due to julia version compat, so
-            # must pass `julia_version=nothing`). In this case, we always fully
-            # specify versions, but if we don't, it's okay to just give us whatever
-            # the resolver prefers
-            Pkg.add([
-                PackageSpec(;name="OpenBLAS_jll",  version=v"0.3.13"),
-                PackageSpec(;name="libblastrampoline_jll", version=v"5.1.1"),
-            ]; julia_version=nothing)
-            @test v"0.3.14" > Pkg.dependencies()[OpenBLAS_jll_UUID].version >= v"0.3.13"
-            @test v"5.1.2" > Pkg.dependencies()[libblastrampoline_jll_UUID].version >= v"5.1.1"
+            @testset "stdlib add" begin
+                Pkg.activate(temp = true)
+                # Stdlib add (impossible constraints due to julia version compat, so
+                # must pass `julia_version=nothing`). In this case, we always fully
+                # specify versions, but if we don't, it's okay to just give us whatever
+                # the resolver prefers
+                Pkg.add(
+                    [
+                        PackageSpec(; name = "OpenBLAS_jll", version = v"0.3.13+0"),
+                        PackageSpec(; name = "libblastrampoline_jll", version = v"5.1.1+0"),
+                    ]; julia_version = nothing
+                )
+                @test v"0.3.14" > Pkg.dependencies()[OpenBLAS_jll_UUID].version >= v"0.3.13"
+                @test v"5.1.2" > Pkg.dependencies()[libblastrampoline_jll_UUID].version >= v"5.1.1"
+            end
+            @testset "non-stdlib JLL add" begin
+                platform = Platform("x86_64", "linux"; libc = "musl")
+                # specific version vs. compat spec
+                @testset for version in (v"3.24.3+0", "3.24.3")
+                    dependencies = [PackageSpec(; name = "CMake_jll", version = version)]
+                    @testset "with context (using private Pkg.add method)" begin
+                        Pkg.activate(temp = true)
+                        ctx = Pkg.Types.Context(; julia_version = nothing)
+                        mydeps = deepcopy(dependencies)
+                        foreach(Pkg.API.handle_package_input!, mydeps)
+                        Pkg.add(ctx, mydeps; platform)
+                    end
+                    @testset "with julia_version" begin
+                        Pkg.activate(temp = true)
+                        Pkg.add(deepcopy(dependencies); platform, julia_version = nothing)
+                    end
+                end
+            end
+
+            @testset "Artifacts stdlib never falls back to registry" begin
+                # Test that when resolving for Julia 1.10 (where Artifacts is a stdlib with version=nothing),
+                # Pkg never installs the external Artifacts v1.3.0 from the registry
+                Pkg.activate(temp = true)
+                # Add a package that depends on Artifacts with julia_version = v"1.10"
+                # Artifacts should remain a stdlib, not be resolved to v1.3.0 from registry
+                ctx = Pkg.Types.Context(; julia_version = v"1.10")
+                # GMP_jll for Julia 1.10 should bring in Artifacts as a dependency
+                Pkg.add(ctx, [PackageSpec(; name = "GMP_jll")])
+
+                # Check that Artifacts is not in the manifest as an external package
+                # (If it were incorrectly resolved from registry, it would appear with version v1.3.0)
+                artifacts_uuid = Base.UUID("56f22d72-fd6d-98f1-02f0-08ddc0907c33")
+                manifest_entry = get(ctx.env.manifest, artifacts_uuid, nothing)
+                if manifest_entry !== nothing
+                    # Artifacts should not have v1.3.0 (the registry version)
+                    @test manifest_entry.version != v"1.3.0"
+                end
+            end
         end
         HistoricalStdlibVersions.unregister!()
     end
